// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
// This code is based on the CodeMirror mergeview.js source:
// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: http://codemirror.net/LICENSE
'use strict';
import { StateEffect, StateField, RangeSetBuilder, RangeSet, } from '@codemirror/state';
import { EditorView, Decoration, WidgetType, GutterMarker, gutter, } from '@codemirror/view';
import { Widget, Panel } from '@lumino/widgets';
import { DecisionStringDiffModel } from '../merge/model';
import { offsetToPos, posToOffset } from '../diff/range';
import { lineToNormalChunks } from '../chunking';
import { EditorWidget, createEditorFactory, } from './editor';
import { valueIn, hasEntries, splitLines } from './util';
const PICKER_SYMBOL = '\u27ad';
const CONFLICT_MARKER = '\u26A0';
export var DIFF_OP;
(function (DIFF_OP) {
    DIFF_OP[DIFF_OP["DIFF_DELETE"] = -1] = "DIFF_DELETE";
    DIFF_OP[DIFF_OP["DIFF_INSERT"] = 1] = "DIFF_INSERT";
    DIFF_OP[DIFF_OP["DIFF_EQUAL"] = 0] = "DIFF_EQUAL";
})(DIFF_OP || (DIFF_OP = {}));
export var EventDirection;
(function (EventDirection) {
    EventDirection[EventDirection["INCOMING"] = 0] = "INCOMING";
    EventDirection[EventDirection["OUTGOING"] = 1] = "OUTGOING";
})(EventDirection || (EventDirection = {}));
const GUTTER_PICKER_CLASS = 'jp-Merge-gutter-picker';
const GUTTER_CONFLICT_CLASS = 'jp-Merge-gutter-conflict';
const CHUNK_CONFLICT_CLASS = 'jp-Merge-conflict';
const conflictDecoration = Decoration.line({ class: CHUNK_CONFLICT_CLASS });
const lineHighlightTypeList = ['chunk', 'conflict', 'start', 'end'];
var Private;
(function (Private) {
    function buildEditorDecorationDict(editorType, chunkAction) {
        const suffix = chunkAction ? '-' + chunkAction : '';
        const prefix = 'cm-merge' + '-' + editorType;
        const dict = {
            chunk: Decoration.line({ class: prefix + '-chunk' + suffix }),
            start: Decoration.line({
                class: prefix + '-chunk' + '-' + 'start' + suffix,
            }),
            end: Decoration.line({ class: prefix + '-chunk' + '-' + 'end' + suffix }),
            endEmpty: Decoration.line({
                class: prefix + '-chunk' + '-' + 'end' + suffix + '-empty',
            }),
            conflict: conflictDecoration,
            inserted: Decoration.mark({ class: prefix + '-' + 'inserted' }),
            deleted: Decoration.mark({ class: prefix + '-' + 'deleted' }),
        };
        return dict;
    }
    Private.buildEditorDecorationDict = buildEditorDecorationDict;
})(Private || (Private = {}));
const mergeViewDecorationDict = {
    left: Private.buildEditorDecorationDict('l'),
    right: Private.buildEditorDecorationDict('r'),
    localMerge: Private.buildEditorDecorationDict('m', 'local'),
    remoteMerge: Private.buildEditorDecorationDict('m', 'remote'),
    customMerge: Private.buildEditorDecorationDict('m', 'custom'),
    eitherMerge: Private.buildEditorDecorationDict('m', 'either'),
    mixedMerge: Private.buildEditorDecorationDict('m', 'mixed'),
};
function getCommonEditorExtensions() {
    return [
        gutterMarkerField,
        highlightField,
        paddingWidgetField,
        pickerLineChunkMappingField,
        conflictMarkerLineChunkMappingField,
    ];
}
function applyMapping({ from, to }, mapping) {
    const map = { from: mapping.mapPos(from), to: mapping.mapPos(to) };
    return map;
}
/**
 * Effect for adding highlighting on lines or on characters
 */
const addHighlightEffect = StateEffect.define({
    map: applyMapping,
});
/**
 * Effect for removing highlighting on lines or on characters
 */
const removeHighlightEffect = StateEffect.define({
    map: applyMapping,
});
/**
 * StateField storing information about highlighting elements of an editor
 */
const highlightField = StateField.define({
    create() {
        return Decoration.none;
    },
    update(highlightRanges, transaction) {
        highlightRanges = highlightRanges.map(transaction.changes);
        for (let e of transaction.effects) {
            let decoration;
            if (e.is(addHighlightEffect)) {
                decoration =
                    mergeViewDecorationDict[e.value.decorationKey][e.value.highlightType];
                highlightRanges = highlightRanges.update({
                    add: [decoration.range(e.value.from, e.value.to)],
                });
            }
            if (e.is(removeHighlightEffect)) {
                decoration =
                    mergeViewDecorationDict[e.value.decorationKey][e.value.highlightType];
                highlightRanges = highlightRanges.update({
                    filter: (from, to, value) => {
                        return decoration.spec.class !== value.spec.class;
                    },
                });
            }
        }
        return highlightRanges;
    },
    provide: field => EditorView.decorations.from(field),
});
/**
 * StateField storing information about padding widgets used to keep the alignment between different editors
 */
export const replacePaddingWidgetEffect = StateEffect.define({
    map: (value, mapping) => value.map(mapping),
});
/**
 * StateField storing information about padding widgets used to keep the alignment between different editors
 */
export const paddingWidgetField = StateField.define({
    create: () => Decoration.none,
    update: (paddingWidgetRanges, transaction) => {
        for (let e of transaction.effects) {
            if (e.is(replacePaddingWidgetEffect)) {
                return e.value;
            }
        }
        return paddingWidgetRanges.map(transaction.changes);
    },
    provide: field => EditorView.decorations.from(field),
});
/**
 * Widget used for aligning lines of different editors that need to be at the same distance from the top of the document
 */
class PaddingWidget extends WidgetType {
    constructor(size) {
        super();
        this.size = size;
    }
    toDOM() {
        let elt = document.createElement('div');
        elt.className = 'cm-merge-spacer';
        elt.style.height = this.size + 'px';
        elt.style.minWidth = '1px';
        return elt;
    }
}
/**
 * Effect for adding a gutter marker
 */
const addGutterMarkerEffect = StateEffect.define({
    map: (val, mapping) => ({
        pos: mapping.mapPos(val.pos),
        on: val.on,
        type: val.type,
    }),
});
/**
 * Effect for removing a gutter marker
 */
const removeGutterMarkerEffect = StateEffect.define({
    map: val => ({ type: val.type }),
});
/**
 * StateField storing information about gutter markers (picker and conflict ones)
 */
const gutterMarkerField = StateField.define({
    create: () => {
        return RangeSet.empty;
    },
    update: (gutters, transaction) => {
        gutters = gutters.map(transaction.changes);
        for (let e of transaction.effects) {
            if (e.is(addGutterMarkerEffect)) {
                if (e.value.on) {
                    const marker = e.value.type === 'picker' ? pickerMarker : conflictMarker;
                    gutters = gutters.update({ add: [marker.range(e.value.pos)] });
                }
            }
            if (e.is(removeGutterMarkerEffect)) {
                gutters = RangeSet.empty;
            }
        }
        return gutters;
    },
});
/**
 * Picker gutter marker DOM Element ➭
 */
const pickerMarker = new (class extends GutterMarker {
    toDOM() {
        let pickerMarker = elt('div', PICKER_SYMBOL);
        pickerMarker.className = GUTTER_PICKER_CLASS;
        return pickerMarker;
    }
})();
/**
 * Conflict gutter marker DOM Element ⚠
 */
const conflictMarker = new (class extends GutterMarker {
    toDOM() {
        let conflictMarker = elt('div', CONFLICT_MARKER);
        conflictMarker.className = GUTTER_CONFLICT_CLASS;
        return conflictMarker;
    }
})();
/**
 * Effect for adding a mapping between a line and a chunk
 * This is used for adding the gutters at the right place : there is a single gutter marker associated with each chunk
 */
const addLineChunkMappingEffect = StateEffect.define({
    map: (val, mapping) => ({
        line: mapping.mapPos(val.line),
        chunk: val.chunk,
        type: val.type,
    }),
});
/**
 * Effect for removing a mapping between a line and a chunk
 * This is used for removing the gutters from the right place there is a single gutter marker associated with each chunk
 */
const removeLineChunkMappingEffect = StateEffect.define({
    map: val => ({ type: val.type }),
});
/**
 * StateField storing information about the mapping between a line and a chunk for picker gutter markers
 */
const pickerLineChunkMappingField = StateField.define({
    create: () => {
        return new Map();
    },
    update: (lineChunkMapping, transaction) => {
        let newLineChunkMapping = lineChunkMapping;
        for (let e of transaction.effects) {
            if (e.is(addLineChunkMappingEffect) && e.value.type === 'picker') {
                newLineChunkMapping.set(e.value.line, e.value.chunk);
            }
        }
        return newLineChunkMapping;
    },
});
/**
 * StateField storing information about the mapping between a line and a chunk for conflict gutter markers
 */
const conflictMarkerLineChunkMappingField = StateField.define({
    create: () => {
        return new Map();
    },
    update: (lineChunkMapping, transaction) => {
        let newLineChunkMapping = lineChunkMapping;
        for (let e of transaction.effects) {
            if (e.is(addLineChunkMappingEffect) && e.value.type === 'conflict') {
                newLineChunkMapping.set(e.value.line, e.value.chunk);
            }
        }
        return newLineChunkMapping;
    },
});
/**
 * A wrapper view for showing StringDiffModels in a MergeView
 */
export function createNbdimeMergeView(options) {
    const { remote, local, merged, readOnly, factory } = options;
    let opts = {
        remote,
        local,
        merged,
        config: { readOnly },
        factory: factory ?? createEditorFactory(),
    };
    let mergeview = new MergeView(opts);
    let editors = [];
    if (mergeview.left) {
        editors.push(mergeview.left);
    }
    if (mergeview.right) {
        editors.push(mergeview.right);
    }
    if (mergeview.merge) {
        editors.push(mergeview.merge);
    }
    let mimetype = (remote || merged).mimetype;
    if (mimetype) {
        // Set the editor mode to the MIME type.
        for (let e of editors) {
            e.remoteEditorWidget.model.mimeType = mimetype;
        }
        mergeview.base.model.mimeType = mimetype;
    }
    return mergeview;
}
/**
 * Used by MergeView to show diff in a string diff model
 */
export class DiffView {
    constructor(model, type, options) {
        this._model = model;
        this._type = type;
        let remoteValue = this._model.remote || '';
        this._remoteEditorWidget = new EditorWidget({
            ...options,
            value: remoteValue,
        });
    }
    init(baseWidget) {
        this._baseEditorWidget = baseWidget;
        let baseEditor = this._baseEditorWidget.cm;
        let remoteEditor = this._remoteEditorWidget.cm;
        this._lineChunks = this._model.getLineChunks();
        this._chunks = lineToNormalChunks(this._lineChunks);
        this.updateView(baseEditor, remoteEditor);
        this.syncScroll(baseEditor, remoteEditor);
    }
    /**
     Update the highlighting in the views of baseEditor and remoteEditor
     */
    updateView(baseEditor, remoteEditor) {
        this.clearHighlighting(remoteEditor, this._model.additions, this._chunks, DIFF_OP.DIFF_INSERT);
        this.clearHighlighting(baseEditor, this._model.deletions, this._chunks, DIFF_OP.DIFF_DELETE);
        this.updateHighlighting(remoteEditor, this._model.additions, this._chunks, DIFF_OP.DIFF_INSERT);
        this.updateHighlighting(baseEditor, this._model.deletions, this._chunks, DIFF_OP.DIFF_DELETE);
    }
    /**
    Update the chunks once a version has been picked
     */
    syncModel() {
        if (!this.modelInvalid()) {
            return;
        }
        let editor = this.remoteEditorWidget.cm;
        let updatedLineChunks = this._model.getLineChunks();
        let updatedChunks = lineToNormalChunks(updatedLineChunks);
        if (this._model.remote === editor.state.doc.toString()) {
            // Nothing to do except update chunks
            this._lineChunks = updatedLineChunks;
            this._chunks = updatedChunks;
            return;
        }
        let cursor = editor.state.selection.main.head;
        let newLines = splitLines(this._model.remote);
        editor.dispatch({
            changes: {
                from: 0,
                to: editor.state.doc.length,
                insert: newLines.slice(0, newLines.length).join(''),
            },
        }, { selection: { anchor: cursor } });
        this._lineChunks = updatedLineChunks;
        this._chunks = updatedChunks;
    }
    /**
  Add a gap DOM element between 2 editors
   */
    buildGap() {
        let lock = (this._lockButton = elt('div', undefined, 'cm-merge-scrolllock'));
        lock.title = 'Toggle locked scrolling';
        let lockWrap = elt('div', [lock], 'cm-merge-scrolllock-wrap');
        lock.innerHTML = '\u21db&nbsp;&nbsp;\u21da';
        lock.addEventListener('scroll', event => {
            this.setScrollLock(!this._lockScroll);
        });
        let gap = elt('div', [lockWrap], 'cm-merge-gap');
        this._gap = gap;
        return this._gap;
    }
    setScrollLock(val, action) {
        this._lockScroll = val;
        if (this._lockButton) {
            this._lockButton.innerHTML = val
                ? '\u21db\u21da'
                : '\u21db&nbsp;&nbsp;\u21da';
        }
    }
    modelInvalid() {
        return (this._model instanceof DecisionStringDiffModel && this._model.invalid);
    }
    /**
     * Synchronize the scrolling between editors.
     * srcEditor refers to the source editor from which the scrolling is done and listened
     * destEditor is the destination editor whose scrolling is synchronized with the one of srcEditor.
     */
    syncScroll(srcEditor, destEditor) {
        if (this.modelInvalid()) {
            return;
        }
        let srcScroller = srcEditor.scrollDOM;
        let destScroller = destEditor.scrollDOM;
        srcScroller.addEventListener('scroll', event => {
            window.requestAnimationFrame(function () {
                destScroller.scrollLeft = srcScroller.scrollLeft;
            });
        });
        destScroller.addEventListener('scroll', event => {
            window.requestAnimationFrame(function () {
                srcScroller.scrollLeft = destScroller.scrollLeft;
            });
        });
    }
    /**
     * The decorationKey is used to have access to the correct css class associated to a given decoration type
     */
    getDecorationKey(sources) {
        let s = this._type;
        let res = s;
        if (this._type === 'merge') {
            s = sources[0].action;
            res = s + 'Merge';
            if (sources.length > 1) {
                for (let si of sources.slice(1)) {
                    if (si.action !== s) {
                        res = 'mixedMerge';
                        break;
                    }
                }
            }
        }
        return res;
    }
    getConflictState(sources) {
        let conflict = false;
        if (sources.length > 0) {
            for (let s of sources) {
                if (s.decision.conflict) {
                    conflict = true;
                    break;
                }
            }
        }
        return conflict;
    }
    /**
     * Create effects related to gutter markers
     */
    createGutterEffects(editor, chunk, pos, on, type) {
        let effects = [];
        let gutterEffect = addGutterMarkerEffect.of({
            pos: pos,
            on: on,
            type: type,
        });
        effects.push(gutterEffect);
        effects.push(addLineChunkMappingEffect.of({
            line: offsetToPos(editor.state.doc, pos).line,
            chunk: chunk,
            type: type,
        }));
        return effects;
    }
    /**
     * Build line background effects and gutter markers effects
     */
    buildLineEffects(editor, chunkArray) {
        let effects = [];
        let isbaseEditor = editor === this._baseEditorWidget.cm;
        for (let chunk of chunkArray) {
            let decorationKey = this.getDecorationKey(chunk.sources);
            let conflict = this.getConflictState(chunk.sources);
            let chunkFirstLine;
            let chunkLastLine;
            if (isbaseEditor) {
                chunkFirstLine = chunk.baseFrom;
                chunkLastLine = chunk.baseTo;
            }
            else {
                chunkFirstLine = chunk.remoteFrom;
                chunkLastLine = chunk.remoteTo;
            }
            for (let i = chunkFirstLine; i < chunkLastLine; i++) {
                let pos = { line: i, column: 0 };
                let startingOffset = posToOffset(editor.state.doc, pos);
                effects.push(addHighlightEffect.of({
                    from: startingOffset,
                    to: startingOffset,
                    highlightType: 'chunk',
                    decorationKey: decorationKey,
                }));
                if (conflict) {
                    effects.push(addHighlightEffect.of({
                        from: startingOffset,
                        to: startingOffset,
                        highlightType: 'conflict',
                        decorationKey: decorationKey,
                    }));
                }
                if (i === chunkFirstLine) {
                    effects.push(addHighlightEffect.of({
                        from: startingOffset,
                        to: startingOffset,
                        highlightType: 'start',
                        decorationKey: decorationKey,
                    }));
                    if (!decorationKey.includes('Merge')) {
                        // For all editors except merge editor, add a picker button
                        effects = effects.concat(this.createGutterEffects(editor, chunk, startingOffset, true, 'picker'));
                    }
                    else if (editor === this._baseEditorWidget.cm) {
                        for (let s of chunk.sources) {
                            if (s.decision.action === 'custom' &&
                                !hasEntries(s.decision.localDiff) &&
                                !hasEntries(s.decision.remoteDiff)) {
                                // We have a custom decision, add picker on base only!*/
                                effects = effects.concat(this.createGutterEffects(editor, chunk, startingOffset, true, 'picker'));
                            }
                        }
                    }
                    else if (conflict && editor === this.remoteEditorWidget.cm) {
                        effects = effects.concat(this.createGutterEffects(editor, chunk, startingOffset, true, 'conflict'));
                    }
                }
                if (i === chunkLastLine - 1) {
                    effects.push(addHighlightEffect.of({
                        from: startingOffset,
                        to: startingOffset,
                        highlightType: 'end',
                        decorationKey: decorationKey,
                    }));
                }
            }
            if (chunkFirstLine === chunkLastLine) {
                const startingOffset = posToOffset(editor.state.doc, {
                    line: chunkFirstLine,
                    column: 0,
                });
                effects.push(addHighlightEffect.of({
                    from: startingOffset,
                    to: startingOffset,
                    highlightType: 'endEmpty',
                    decorationKey: decorationKey,
                }));
                if (!decorationKey.includes('Merge')) {
                    effects = effects.concat(this.createGutterEffects(editor, chunk, chunkLastLine, true, 'picker'));
                }
                else if (conflict) {
                    effects = effects.concat(this.createGutterEffects(editor, chunk, startingOffset, true, 'picker'));
                }
            }
        }
        return effects;
    }
    /**
     * Remove line background effects and gutter markers effects
     */
    clearLineEffects(editor, chunkArray) {
        let effects = [];
        for (let chunk of chunkArray) {
            let sources = chunk.sources;
            let decorationKey = this.getDecorationKey(sources);
            for (let highlightType of lineHighlightTypeList) {
                effects.push(removeHighlightEffect.of({
                    highlightType: highlightType,
                    decorationKey: decorationKey,
                }));
            }
        }
        if (editor !== this._baseEditorWidget.cm) {
            effects.push(removeGutterMarkerEffect.of({ type: 'all' }));
            effects.push(removeLineChunkMappingEffect.of({ type: 'picker' }));
            effects.push(removeLineChunkMappingEffect.of({ type: 'conflict' }));
        }
        return effects;
    }
    /**
     * Build character highlighting effects
     */
    buildCharacterHighlighting(editor, diffRanges, markType) {
        const effects = [];
        const sources = [];
        if (markType === DIFF_OP.DIFF_INSERT || markType === DIFF_OP.DIFF_DELETE) {
            const highlightType = markType === DIFF_OP.DIFF_DELETE ? 'deleted' : 'inserted';
            for (let r of diffRanges) {
                if (r.source !== undefined) {
                    sources.push(r.source);
                }
                const startingOffset = posToOffset(editor.state.doc, {
                    line: r.from.line,
                    column: r.from.column,
                });
                const endingOffset = posToOffset(editor.state.doc, {
                    line: r.to.line,
                    column: r.to.column,
                });
                effects.push(addHighlightEffect.of({
                    from: startingOffset,
                    to: endingOffset,
                    highlightType: highlightType,
                    decorationKey: this.getDecorationKey(sources),
                }));
            }
        }
        return effects;
    }
    /**
    Clear character highlighting effects
    */
    clearCharacterHighlighting(editor, diffRanges, markType) {
        let effects = [];
        let sources = [];
        if (markType === DIFF_OP.DIFF_INSERT || markType === DIFF_OP.DIFF_DELETE) {
            let highlightType = markType === DIFF_OP.DIFF_DELETE ? 'deleted' : 'inserted';
            for (let r of diffRanges) {
                if (r.source !== undefined) {
                    sources.push(r.source);
                }
                let decorationKey = this.getDecorationKey(sources);
                effects.push(removeHighlightEffect.of({
                    highlightType: highlightType,
                    decorationKey: decorationKey,
                }));
            }
        }
        return effects;
    }
    /**
    Build line effects (highlight and gutters) and character highlighting effects and dispatch them
     */
    updateHighlighting(editor, diffRanges, chunkArray, type) {
        const LineHighlightEffects = this.buildLineEffects(editor, chunkArray);
        const MarkHighlightEffects = this.buildCharacterHighlighting(editor, diffRanges, type);
        const effects = LineHighlightEffects.concat(MarkHighlightEffects);
        editor.dispatch({ effects });
    }
    /**
    Clear line effects (highlight and gutters) and character highlighting effects and dispatch them
     */
    clearHighlighting(editor, diffRanges, chunkArray, type) {
        const clearLineEffects = this.clearLineEffects(editor, chunkArray);
        const clearCharacterHighlightEffects = this.clearCharacterHighlighting(editor, diffRanges, type);
        const effects = clearLineEffects.concat(clearCharacterHighlightEffects);
        editor.dispatch({ effects });
    }
    get remoteEditorWidget() {
        return this._remoteEditorWidget;
    }
    get baseEditorWidget() {
        return this._baseEditorWidget;
    }
    get lineChunks() {
        return this._lineChunks;
    }
    get model() {
        return this._model;
    }
}
/**
 * From a line in base, find the matching line in another editor by line chunks
 *
 */
function getMatchingEditLineLC(toMatch, chunks) {
    let editLine = toMatch.baseFrom;
    for (let i = 0; i < chunks.length; ++i) {
        let chunk = chunks[i];
        if (chunk.baseFrom === editLine) {
            return chunk.remoteTo;
        }
        if (chunk.baseFrom > editLine) {
            break;
        }
    }
    return toMatch.baseTo;
}
/**
 * Find which line numbers align with each other, in the
 * set of DiffViews. The returned array is of the format:
 *
 * [ aligned line #1:[Edit line number, (DiffView#1 line number, DiffView#2 line number,) ...],
 *   aligned line #2 ..., etc.]
 */
function findAlignedLines(dvs) {
    let linesToAlign = [];
    let ignored = [];
    // First fill directly from first DiffView
    let dv = dvs[0];
    let others = dvs.slice(1);
    for (let i = 0; i < dv.lineChunks.length; i++) {
        let chunk = dv.lineChunks[i];
        let lines = [chunk.baseTo, chunk.remoteTo];
        for (let o of others) {
            lines.push(getMatchingEditLineLC(chunk, o.lineChunks));
        }
        if (linesToAlign.length > 0 &&
            linesToAlign[linesToAlign.length - 1][0] === lines[0]) {
            let last = linesToAlign[linesToAlign.length - 1];
            for (let j = 0; j < lines.length; ++j) {
                last[j] = Math.max(last[j], lines[j]);
            }
        }
        else {
            if (linesToAlign.length > 0) {
                let prev = linesToAlign[linesToAlign.length - 1];
                let diff = lines[0] - prev[0];
                for (let j = 1; j < lines.length; ++j) {
                    if (diff !== lines[j] - prev[j]) {
                        diff = null;
                        break;
                    }
                }
                if (diff === null) {
                    linesToAlign.push(lines);
                }
                else {
                    ignored.push(lines[0]);
                    continue;
                }
            }
            else {
                linesToAlign.push(lines);
            }
        }
    }
    // Then fill any chunks from remaining DiffView, which are not already added
    for (let o = 0; o < others.length; o++) {
        for (let i = 0; i < others[o].lineChunks.length; i++) {
            let chunk = others[o].lineChunks[i];
            // Check against existing matches to see if already consumed:
            let j = 0;
            for (; j < linesToAlign.length; j++) {
                let align = linesToAlign[j];
                if (valueIn(chunk.baseTo, ignored)) {
                    // Chunk already consumed, continue to next chunk
                    j = -1;
                    break;
                }
                else if (align[0] >= chunk.baseTo) {
                    // New chunk, which should be inserted in pos j,
                    // such that linesToAlign are sorted on edit line
                    break;
                }
            }
            if (j > -1) {
                let lines = [chunk.baseTo, getMatchingEditLineLC(chunk, dv.lineChunks)];
                for (let k = 0; k < others.length; k++) {
                    if (k === o) {
                        lines.push(chunk.remoteTo);
                    }
                    else {
                        lines.push(getMatchingEditLineLC(chunk, others[k].lineChunks));
                    }
                }
                if (linesToAlign.length > j && linesToAlign[j][0] === chunk.baseTo) {
                    let last = linesToAlign[j];
                    for (let k = 0; k < lines.length; ++k) {
                        last[k] = Math.max(last[k], lines[k]);
                    }
                }
                else {
                    linesToAlign.splice(j, 0, lines);
                }
            }
        }
    }
    return linesToAlign;
}
// Merge view, containing 1 or 2 diff views.
export class MergeView extends Panel {
    constructor(options) {
        super();
        this._measuring = -1;
        let remote = options.remote;
        let local = options.local || null;
        let merged = options.merged || null;
        let left = (this._left = null);
        let right = (this._right = null);
        let merge = (this._merge = null);
        let panes = 0;
        this._diffViews = [];
        let main = options.remote || options.merged;
        if (!main) {
            throw new Error('Either remote or merged model needs to be specified!');
        }
        const value = main.base !== null ? main.base : main.remote ?? '';
        // Whether merge view should be readonly
        let readOnly = options.config?.readOnly ?? false;
        options.config = {
            ...options.config,
            lineNumbers: options.config?.lineNumbers !== false,
            // For all others:
            readOnly: true,
        };
        if (merged) {
            // Turn off linewrapping for merge view by default, keep for diff
            options.config.lineWrap = options.config.lineWrap ?? false;
        }
        this._aligning = true;
        /**
         * Listener extension to track for changes in the editorView
         */
        const listener = EditorView.updateListener.of(update => {
            if (this._measuring < 0 &&
                /*update.heightChanged || */ update.viewportChanged &&
                !update.transactions.some(tr => tr.effects.some(e => e.is(replacePaddingWidgetEffect)))) {
                this.alignViews();
            }
        });
        /**
         * MergeControlGutter to set the gutter in a given editor and the mousedown events for picker markers
         */
        const mergeControlGutter = [
            gutterMarkerField,
            gutter({
                class: 'cm-gutter',
                markers: editor => editor.state.field(gutterMarkerField),
                initialSpacer: () => pickerMarker,
                domEventHandlers: {
                    mousedown: (editor, line) => {
                        this.onGutterClick(editor, line);
                        return true;
                    },
                },
            }),
        ];
        /*
         * Different cases possible:
         *   - Local and merged supplied: Merge:
         *     - Always use left, right and merge panes
         *     - Use base if `showBase` not set to false
         *   - Only remote supplied: Diff:
         *     - No change: Use ony base editor
         *     - Entire content added/deleted: Use only base editor,
         *       but with different classes
         *     - Partial changes: Use base + right editor
         */
        this._base = new EditorWidget({
            ...options,
            extensions: [
                ...(options.extensions ?? []),
                listener,
                mergeControlGutter,
                getCommonEditorExtensions(),
            ],
            value,
        });
        // START MERGE CASE
        if (merged) {
            this._gridPanel = new Panel();
            this.addWidget(this._gridPanel);
            this._gridPanel.addClass('cm-merge-grid-panel');
            let showBase = options.showBase !== false;
            if (!showBase) {
                this._base.node.style.display = 'hidden';
            }
            let leftWidget;
            if (!local || local.remote === null) {
                // Local value was deleted
                left = this._left = null;
                leftWidget = new Widget({
                    node: elt('div', 'Value missing', 'jp-mod-missing'),
                });
            }
            else {
                left = this._left = new DiffView(local, 'left', {
                    ...options,
                    // Copy configuration
                    config: { ...options.config },
                    extensions: [
                        ...(options.extensions ?? []),
                        listener,
                        mergeControlGutter,
                        getCommonEditorExtensions(),
                    ],
                });
                this._diffViews.push(left);
                leftWidget = left.remoteEditorWidget;
            }
            this._gridPanel.addWidget(leftWidget);
            leftWidget.addClass('cm-merge-left-editor');
            if (showBase) {
                this._gridPanel.addWidget(this._base);
                this._base.addClass('cm-central-editor');
            }
            let rightWidget;
            if (!remote || remote.remote === null) {
                // Remote value was deleted
                right = this._right = null;
                rightWidget = new Widget({
                    node: elt('div', 'Value missing', 'jp-mod-missing'),
                });
            }
            else {
                right = this._right = new DiffView(remote, 'right', {
                    ...options,
                    // Copy configuration
                    config: { ...options.config },
                    extensions: [
                        ...(options.extensions ?? []),
                        listener,
                        mergeControlGutter,
                        getCommonEditorExtensions(),
                    ],
                });
                this._diffViews.push(right);
                rightWidget = right.remoteEditorWidget;
            }
            this._gridPanel.addWidget(rightWidget);
            rightWidget.addClass('cm-merge-pane');
            rightWidget.addClass('cm-merge-right-editor');
            merge = this._merge = new DiffView(merged, 'merge', {
                ...options,
                // Copy configuration
                config: { ...options.config, readOnly },
                extensions: [
                    ...(options.extensions ?? []),
                    listener,
                    mergeControlGutter,
                    getCommonEditorExtensions(),
                ],
            });
            this._diffViews.push(merge);
            let mergeWidget = merge.remoteEditorWidget;
            this._gridPanel.addWidget(mergeWidget);
            mergeWidget.addClass('cm-merge-editor');
            //END MERGE CASE
            panes = 3 + (showBase ? 1 : 0);
            // START DIFF CASE
        }
        else if (remote) {
            this._gridPanel = new Panel();
            this.addWidget(this._gridPanel);
            this._gridPanel.addClass('cm-single-panel');
            // If in place for type guard
            this._gridPanel.addWidget(this._base);
            if (remote.unchanged || remote.added || remote.deleted) {
                if (remote.unchanged) {
                    this._base.addClass('cm-merge-pane-unchanged');
                }
                else if (remote.added) {
                    this._base.addClass('cm-merge-pane-added');
                }
                else if (remote.deleted) {
                    this._base.addClass('cm-merge-pane-deleted');
                }
                panes = 1;
            }
            else {
                this._gridPanel = new Panel();
                this.addWidget(this._gridPanel);
                this._gridPanel.addClass('cm-diff-grid-panel');
                // If in place for type guard
                this._gridPanel.addWidget(this._base);
                this._base.addClass('cm-diff-left-editor');
                right = this._right = new DiffView(remote, 'right', {
                    ...options,
                    // Copy configuration
                    config: { ...options.config },
                    extensions: [
                        ...(options.extensions ?? []),
                        listener,
                        mergeControlGutter,
                        getCommonEditorExtensions(),
                    ],
                });
                this._diffViews.push(right);
                let rightWidget = right.remoteEditorWidget;
                rightWidget.addClass('cm-diff-right-editor');
                this.addWidget(new Widget({ node: right.buildGap() }));
                this._gridPanel.addWidget(rightWidget);
                panes = 2;
            }
        }
        this.addClass('cm-merge-' + panes + 'pane');
        for (let dv of [left, right, merge]) {
            if (dv) {
                dv.init(this._base);
            }
        }
        this._aligning = false;
        if (this._diffViews.length > 0) {
            this.scheduleAlignViews();
        }
    }
    /**
     * Align the matching lines of the different editors
     */
    alignViews() {
        let lineHeight = this._base.cm.defaultLineHeight;
        if (this._aligning) {
            return;
        }
        this._aligning = true;
        // Find matching lines
        let linesToAlign = findAlignedLines(this._diffViews);
        // Function modifying DOM to perform alignment:
        let self = this;
        let editors = [self.base.cm];
        let builders = [];
        for (let dv of self._diffViews) {
            editors.push(dv.remoteEditorWidget.cm);
        }
        for (let i = 0; i < editors.length; i++) {
            builders.push(new RangeSetBuilder());
        }
        let sumDeltas = new Array(editors.length).fill(0);
        let nLines = editors.map(editor => editor.state.doc.lines);
        for (let alignment_ of linesToAlign) {
            let alignment = alignment_.slice(0, 3);
            let lastLine = Math.max(...alignment);
            let lineDeltas = alignment.map((line, i) => lastLine - line - sumDeltas[i]);
            // If some paddings will be before the current line, it means all other editors
            // must add a padding.
            let minDelta = Math.min(...lineDeltas);
            let correctedDeltas = lineDeltas.map(line => line - minDelta);
            correctedDeltas.forEach((delta, i) => {
                let side = -1;
                let line = alignment[i];
                if (delta > 0 && line < nLines[i]) {
                    sumDeltas[i] += delta;
                    let offset = posToOffset(editors[i].state.doc, {
                        line,
                        column: 0,
                    });
                    builders[i].add(offset, offset, Decoration.widget({
                        widget: new PaddingWidget(delta * lineHeight),
                        block: true,
                        side,
                    }));
                }
            });
        }
        // Padding at the last line of the editor
        let totalHeight = nLines.map((line, i) => line + sumDeltas[i]);
        let maxHeight = Math.max(...totalHeight);
        totalHeight.slice(0, 3).forEach((line, i) => {
            if (maxHeight > line) {
                let end = editors[i].state.doc.length;
                let delta = maxHeight - line;
                sumDeltas[i] += delta;
                builders[i].add(end, end, Decoration.widget({
                    widget: new PaddingWidget(delta * lineHeight),
                    block: true,
                    side: 1,
                }));
            }
        });
        for (let i = 0; i < editors.length; i++) {
            let decoSet = builders[i].finish();
            if (!RangeSet.eq([decoSet], [editors[i].state.field(paddingWidgetField)])) {
                editors[i].dispatch({
                    effects: replacePaddingWidgetEffect.of(decoSet),
                });
            }
            console.log('after addition of paddings, number of lines is:', editors[i].state.doc.lines);
        }
        this._aligning = false;
    }
    /**
     * Used to schedule the call of alignViews
     */
    scheduleAlignViews() {
        if (this._measuring < 0) {
            let win = this._gridPanel.node.ownerDocument.defaultView || window;
            this._measuring = win.requestAnimationFrame(() => {
                this._measuring = -1;
                this.alignViews();
            });
        }
    }
    getMergedValue() {
        if (!this.merge) {
            throw new Error('No merged value; missing "merged" view');
        }
        return this.merge.remoteEditorWidget.doc.toString();
    }
    /**
     * Actions and updates performed when a gutter marker is clicked
     */
    onGutterClick(editor, line) {
        let effects = [];
        let offset = line.from;
        let gutterMarkerline = offsetToPos(editor.state.doc, offset).line;
        let isPicker = editor != this.merge?.remoteEditorWidget.cm;
        if (isPicker) {
            let pickerLineChunksMapping = editor.state.field(pickerLineChunkMappingField);
            let chunk = pickerLineChunksMapping.get(gutterMarkerline);
            if (!(editor == this._base.cm)) {
                for (let source of chunk.sources) {
                    source.decision.action = source.action;
                }
            }
            else if (this.merge && editor === this._base.cm) {
                for (let source of chunk.sources) {
                    source.decision.action = 'base';
                }
            }
            for (let i = chunk.sources.length - 1; i >= 0; --i) {
                let source = chunk.sources[i];
                if (this.merge && hasEntries(source.decision.customDiff)) {
                    // Custom diffs are cleared on pick,
                    // as there is no way to re-pick them
                    source.decision.customDiff = [];
                }
            }
            if (chunk.sources.length === 0) {
                // All decisions empty, remove picker
                // In these cases, there should only be one picker, on base
                // so simply remove the one we have here
            }
            effects.push(addGutterMarkerEffect.of({
                pos: line.from,
                on: false,
                type: 'picker',
            }));
        }
        else {
            // conflict picker
            let conflictLineChunksMapping = editor.state.field(conflictMarkerLineChunkMappingField);
            let chunk = conflictLineChunksMapping.get(gutterMarkerline);
            for (let source of chunk.sources) {
                if (editor !== this._base.cm) {
                    source.decision.conflict = false;
                }
            }
        }
        editor.dispatch({ effects: effects });
        this.updateDiffModels();
        this.updateDiffViews();
        this.alignViews();
        return true;
    }
    /**
     * Update of the models of the diffViews by calling syncModel that updates the chunks
     */
    updateDiffModels() {
        for (let dv of this._diffViews) {
            if (dv.model instanceof DecisionStringDiffModel) {
                dv.model.invalidate();
            }
            dv.syncModel();
        }
    }
    /**
     * Update of the views of the diffViews by calling updateView
     * Before updating the diffViews, baseEditor needs to be cleared from its pickers
     */
    updateDiffViews() {
        this.clearBaseEditorPickers();
        for (let dv of this._diffViews) {
            dv.updateView(dv.baseEditorWidget.cm, dv.remoteEditorWidget.cm);
        }
    }
    /**
     * Clear the pickers of the baseEditor
     * The baseEditor is indeed cumulating pickers from different diffViews
     * Since this editor is common to the 3 diffviews
     */
    clearBaseEditorPickers() {
        let effects = [];
        effects.push(removeGutterMarkerEffect.of({ type: 'all' }));
        effects.push(removeLineChunkMappingEffect.of({ type: 'picker' }));
        this._base.cm.dispatch({ effects });
    }
    get left() {
        return this._left;
    }
    get right() {
        return this._right;
    }
    get merge() {
        return this._merge;
    }
    get base() {
        return this._base;
    }
}
/**
 To create DOM element with defined properties
 */
function elt(tag, content, className, style) {
    let e = document.createElement(tag);
    if (className) {
        e.className = className;
    }
    if (style) {
        e.style.cssText = style;
    }
    if (typeof content === 'string') {
        e.appendChild(document.createTextNode(content));
    }
    else if (content) {
        for (let i = 0; i < content.length; ++i) {
            e.appendChild(content[i]);
        }
    }
    return e;
}
//# sourceMappingURL=mergeview.js.map